
;ROUTINE: DRAW BACKDROP					By: Thomas Williamson
;size: 61 bytes, requires STRETCH IT, approx. 102k cycles
;input: de holds start of backdrop image
;destroys: everything
;output: screen display
;stretches a 64*31 pixel image to 128*62...beautifully, I might add
;change the 3rd and 4th line to do a 64*32 stretch
drawbackdrop:
	push de						;put data to be read into ix
	pop ix
	ld hl, $ca20				;start on 3rd line
	ld b, 31					;31 lines
dbdroplp:
	push bc
	ld b, 8						;8 bytes per row
dbdroplp2:
	push bc
	push hl
	ld a, (ix + 0)				;get byte from ix, get the first four bits
	push af
	and %11110000
	call stretchit				;...and stretch it
	ld l, c						;save stretched byte to l
	pop af
	and %00001111				
	rrca
	rrca
	rrca
	rrca
	call stretchit				;now stretch the last four bits
	ld h, c						;store in h, so now we have a 16-bit number
								;stretched from a byte
	pop de			
	ex de, hl					;pop screen pos. into de and flip it into hl, 
								;retaining hl in de
	ld (hl), e
	inc hl
	ld (hl), d					;load the 16 bits
	ld bc, 15
	add hl, bc					;go down one row (already incremented once)
	ld (hl), e
	inc hl
	ld (hl), d					;load the 16 bits
	sbc hl, bc					;go up 15 bytes, right to the next spot
	inc ix						;increase data pointer
	pop bc
	djnz dbdroplp2
	ld bc, 16
	add hl, bc					;on the wrong row, go to the next one
	pop bc
	djnz dbdroplp
	ret
	
;ROUTINE: STRETCH IT					By: Thomas Williamson
;size: 36 bytes, max 160 cycles
;input: 1st 4 bits of a
;destroys: bc, a
;output: stretched a in c
;stretches 4 bits into 8 bits. give it :10110000, it returns: 11001111
;now, see this: give: X.XX.... return: XX..XXXX
stretchit:
	ld c, 0
	ld b, a
	and %00010000
	jr z, nozip1
	ld c, %00000011
nozip1:
	ld a, b
	and %00100000
	jr z, nozip2
	ld a, c
	or %00001100
	ld c, a
nozip2:
	ld a, b
	and %01000000
	jr z, nozip3
	ld a, c
	or %00110000
	ld c, a
nozip3:
	ld a, b
	and %10000000
	ret z
	ld a, c
	or %11000000
	ld c, a
	ret

;ROUTINE: LOAD STRING					;By: Thomas Williamson
;size: 31 bytes
;input: hl contains pointer to var. data
;destroys: ? probably everything
;output: requested file loaded to $8100 (max 8k)
;if the variable isn't there, things get screwy (but not chaotic, as in no crashes)
loadstring:
	rst 20h
	rst 10h
	ld a,b
	ex de,hl
	call _ahl_plus_2_pg3
	call _set_abs_src_addr
	ld a,1
	ld hl,$0100
	call _set_abs_dest_addr
	xor a
	ld hl,$2000
	call _set_mm_num_bytes
	call _mm_ldir
	jp _ram_page_1

;ROUTINE: DRAW SPR 16*24				;By: Thomas Williamson
;size: 68 bytes, avg. 8700 cycles
;input: hl = start of sprite data, de=screen pos.
;destroys: everything
;output: screen display (xorred to screen, ie draw same data on screen agian to erase)
drawgamespr:
	ld a, $a0
	ld (bufferman), a
	jr skipassignbuffer
drawspr2416:
	ld a, $fc
	ld (bufferman), a
skipassignbuffer:
	push hl
	ld h, $00				;code borrowed from findpixel
	ld a,e
	add a,a
	add a,a
	ld l,a
	ld a,d
	rra
	add hl,hl
	rra
	add hl,hl
	rra
	or l
	ld l,a
	ld a, (bufferman)
	ld b, a
	ld a, h
	add a, b
	ld h, a
	ld a,d
	and %00000111
	pop ix
	ld b, 24
	ld c, a
ds21lp:
	push bc
	ld e, (ix + 0)
	ld d, (ix + 1)
	ld a, c
	ld c, 0
	or a
	jr z, skiprotate
	ld b, a
ds21lp2:
	rr e
	rr d
	rr c
	djnz ds21lp2
skiprotate:
	ld a, (hl)
	xor e
	ld (hl), a
	inc hl
	ld a, (hl)
	xor d
	ld (hl), a
	inc hl
	ld a, (hl)
	xor c
	ld (hl), a
	inc ix
	inc ix
	ld bc, 14
	add hl, bc
	pop bc
	djnz ds21lp
	ret
bufferman:
	.db 0

;ROUTINE: FLIP SPRITE					By: Thomas Williamson
;size: 57 bytes
;input: hl at 24*16 sprite location
;destroys: everything
;output: sprite is flipped horizontally
flipsprite:
	ld b, 24
	ld d, %10000000
	ld e, %00000001
fspriteloop:
	push bc
	push hl
	ld c, (hl)
	inc hl
	ld h, (hl)
	ld b, 8
	ld l, 0
fmeloop:
	ld a, c
	and d
	jr z, nobitflip
	ld a, l
	or e
	ld l, a
nobitflip
	rrc d
	rlc e
	djnz fmeloop
	ld c, h
	ld h, l
	ld l, b
	ld b, 8
fmeloop2:
	ld a, c
	and d
	jr z, nobitflip2
	ld a, l
	or e
	ld l, a
nobitflip2
	rrc d
	rlc e
	djnz fmeloop2
	push hl
	pop bc
	pop hl
	ld (hl), c
	inc hl
	ld (hl), b
	inc hl
	pop bc
	djnz fspriteloop
	ret

;clear buffer
clearbuffer:
	ld hl, $a000
	ld de, $a001
	ld (hl), l
	ld bc, 1023
	ldir
	ret

;Copy Screen
copyscreen
	ld hl, $a020
	ld de, $fc20
	ld bc, 992
	ldir
	ret

;Old Routines:
keyloop:
	ld l, 0
keyloopb:
	push bc
	ld b, 10
keyloop2:
	ld e, 128
 	ld a,%1111110
	out (1),a
 	nop
 	in a,(1)
 	rra
 	call nc,ldkey
	rrc e
 	rra
 	call nc,ldkey
	rrc e
 	rra
 	call nc,ldkey
	rrc e
 	rra
 	call nc,ldkey
	rrc e
	ld a,%00111111
	out (1), a
	nop
	in a, (1)
	and %01000000
	call z, ldkey
	rrc e
	in a, (1)
	ld h, a
	and %00100000
	call z, ldkey
	rrc e
	ld a, %01011111
	out (1), a
	nop
	in a, (1)
	push af
	and %10000000
	call z, ldkey
	rrc e
	pop af
	and %01000000
	call z, ldkey
	djnz keyloop2
	pop bc
	djnz keyloopb
	ld a, l
	ld (keyvar), a
	ret
ldkey:
	ld d, a
	ld a, l
	or e
	ld l, a
	ld a, d
	ret
; keys:
; 0=nothing
; 8=down
; 7=left
; 6=right
; 5=up
; 4=exit
; 3=2nd
; 2=alpha			;this bit-1 turned on
; 1=graph

waitkey:
	ld b, 1
	call keyloop
	or a
	jr nz, waitkey
	ret

pause:
	call waitkey
pause3:
	ld b, 1
	call keyloop
	and %00000100
	jr z, pause3
	jp waitkey

drawstring:
	push de
	ld (_pencol), de
	call _vputs
	pop de
	ret

drawcharname:
	ld hl, charnames
drawseriesthing:
	inc a
	ld b, a
ditemloop
	push bc
	ld b, 1
	ld a, 0							;"/"
	cpir
	pop bc
	djnz ditemloop
	jp drawstring

clearselection:
	ld hl, $fc60
	ld de, $fc61
	xor a
	ld (hl), a
	ld bc, 800
	ldir
	ret

; The following was borrowed (and modified to display 'cleaner' numbers) from Jezzball
; thx to Magnus Svedin :)
freakynumber2:
	ld a, l
	cp 232 
	jr nc, drawnumber2
	call lessdigit
	jr drawnumber2
freakynumber:
	ld a, l
	cp $10
	jr nc, drawnumber2
	call lessdigit
	jr drawnumber2
drawnumber:
	ld d, 5									;5 digits
	ld a, h	
	cp 39									;if number is between 9984 and 10240, call special routine
	jr z, freakynumber
	jr nc, drawnumber2
	call lessdigit									;4 digits
	ld a, h
	cp 3									;if number is between 768 and 1024, call special routine
	jr z, freakynumber2
	jr nc, drawnumber2
	call lessdigit
	ld a, l									;3 digits
	cp 100
	jr nc, drawnumber2
	ld a, h									;if number is between 256 and 768, it is 3 digits
	or a
	jr nz, drawnumber2
	call lessdigit								;2 digits
	ld a, l
	cp 10 
	jr nc, drawnumber2
	call lessdigit									;1 digit
drawnumber2:
	ld (_penCol), bc
	ld b, d								;number of digits
	push de
	ld de, numberbuffer+4   ;buffer to store ASCII in
PNStl1:
  call _divHLby10   ;call function to convert from number to digit
  add a, '0'          ;convert digit to ASCII-number for digit
  ld (de), a          ;store in ASCII-buffer
  dec de
  djnz PNStl1         ;loop 5 times
  ld hl, numberbuffer
	pop de
	ld b, d
	ld a, 5
	sub b
	ld d, 0
	ld e, a
	add hl, de
  jp _vputsn   	;print it to screen
lessdigit:
	ld a, c
	add a, 4
	ld c, a
	dec d
	ret

numberbuffer:
	.db  0,0,0,0,0,0

hline:
	add a, a
	add a, a
	ld l, a
	ld h, $3f
	add hl, hl
	add hl, hl
	ld b, 32
hlineloop:
	ld a, r
	ld (hl), a
	inc hl
	djnz hlineloop
	ret

.end
